<!DOCTYPE html>
<head>
	<meta charset="utf-8">
	<meta http-equiv="X-UA-Compatible" content="IE=edge,chrome=1">

    {% block title %}
    {% endblock %}

	<meta name="description" content="Chanjo2 Report: coverage report generator">
	<meta name="author" content="Chiara Rasi">
	<meta name="viewport" content="width=device-width, initial-scale=1.0">
	<meta http-equiv="Content-Security-Policy" content="upgrade-insecure-requests">

	<!-- place 'favicon.ico' + 'apple-touch-icon.png' in the root directory -->
	<link rel="Shortcut Icon"
				href="#"
				type="image/x-icon">

	{% block css %}
		<!-- Compiled and minified CSS -->
		<link href="https://cdn.jsdelivr.net/npm/bootstrap@5.0.2/dist/css/bootstrap.min.css" rel="stylesheet" integrity="sha384-EVSTQN3/azprG1Anm3QDgpJLIm9Nao0Yz1ztcQTwFspd3yD65VohhpuuCOmLASjC" crossorigin="anonymous">

		<!-- Customizations -->
		<link rel="stylesheet" type="text/css" href="{{ url_for('static', path='main.css') }}">
	{% endblock %}


</head>
<body>

	<nav class="navbar navbar-expand-lg navbar-light bg-light">
	  <div class="container">
		  <a class="navbar-brand" href="#">
			  Chanjo2
			  <span class="text-muted" style="font-size: 0.7em;">v{{ software_version }}</span>
		  </a>

		  {% block pdf_export %}
		  {% endblock %}
	  </div><!-- /.container -->
	</nav>

	<div class="container">

        {% block content %}
        {% endblock %}

	</div>

	{% block js_code %}
	<!-- Bootstrap JavaScript bundle-->
	<script src="https://cdn.jsdelivr.net/npm/bootstrap@5.0.2/dist/js/bootstrap.bundle.min.js" integrity="sha384-MrcW6ZMFYlzcLA8Nl+NtUVF0sA7MsXsP1UyJoMp4YLEuNSfAP+JcXn/tWtIaxVXM" crossorigin="anonymous"></script>
	<!-- JQuery CDN-->
	<script src="https://code.jquery.com/jquery-3.7.1.min.js" integrity="sha256-/JqT3SQfawRcv/BIHPThkBvs0OEvtFFmqPF/lYI/Cxo=" crossorigin="anonymous"></script>
	{% endblock %}
</body>
</html>